<?php
/*
	squash.inc
	Roberto Machorro <rmach@ciberlinux.net>

	This is the main library file for Squash. By including this file
	in your HTML/PHP scripts, you will be able to access the database
	and view/modify/add the data in it.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
*/

include "../lib/config.inc";

/*
	Connectivity functions.
	Used to open/close a connection to the database, should be standard
	in all routines, as these will provide the handle for the queries.
	Although PHP automatically closes all DB handles, closing manually
	doesn't hurt...
*/

function connect_squash_dbms() {
	global $DBMS_HOSTNAME, $USER_NAME, $USER_PASSWORD;
	global $MESSAGE_DBCONNECT_ERROR, $MESSAGE_DBQUERY_ERROR;
	global $DATABASE_NAME;

	$sqlid = mysql_connect($DBMS_HOSTNAME, $USER_NAME, $USER_PASSWORD) or
				die ($MESSAGE_DBCONNECT_ERROR);
	$res = mysql_query("use $DATABASE_NAME", $sqlid) or
				die ($MESSAGE_DBQUERY_ERROR);

	return $sqlid;
}

function close_squash_dbms($sqlid) {
	return mysql_close($sqlid);
}

/*
	SQL General purpose command functions.
	These allow the system to query the database and retrieve tables,
	you must know that these are terribly biased towards MySQL... =P
	if you are to use another DBMS, it would be just a matter of
	changing here to the appropriate functions. The Squash id
	function can also be tailored to work with propietery Id counter
	mechanisms that mey belong to some DBMS's (like Hughe's mSQL)
	change to your taste and you may even get rid of race conditions ;)
*/

function query_squash($query, $sqlid) {
	global $MESSAGE_DBQUERY_ERROR;

	$sql_result = mysql_query($query, $sqlid) or
				die ($MESSAGE_DBQUERY_ERROR);

	return $sql_result;
}

function num_squash_rows($sql_result) {
	return mysql_num_rows($sql_result);
}

function affected_squash_rows($sql_result) {
	return mysql_affected_rows($sql_result);
}

function fetch_squash_row($sql_result) {
	return mysql_fetch_row($sql_result);
}

function get_next_squash_id($table, $sqlid) {
	/*
		NOTE: race conditions may exist here, hehehe... fix!
	*/
	$res = query_squash("
				SELECT id
				FROM $table
				ORDER BY id DESC",
				$sqlid);
	if (num_squash_rows($res) > 0) {
		$row = fetch_squash_row($res);
		return ($row[0] + 1);
	}
	return 0;
}

/*
	Squash User related functions.
	All these handle the use/registration of users and their status/
	level/validations. There is also a password encryption function
	which can be adapted to suit the cryptographic needs of each
	site, in the default case we are just happy by using MySQL's
	default password crypt routine (yep, we actually call MySQL to
	crypt the password and return its result... hehehe...)
*/

function crypt_squash_password($pass, $sqlid) {
	$res = query_squash("SELECT password('$pass')", $sqlid);
	$row = fetch_squash_row($res);
	return $row[0];
}

function squash_user_exists($nick, $sqlid) {
	$res = query_squash("
				SELECT id
				FROM people
				WHERE nick = '$nick'",
				$sqlid);
	if (num_squash_rows($res) > 0) {
		return true;
	}
	return false;
}

function create_squash_user($nick, $pass, $name, $email, $permits, $sqlid) {
	$epass = crypt_squash_password($pass, $sqlid);
	$id = get_next_squash_id("people", $sqlid);
	$res = query_squash("
				INSERT INTO people
				VALUES ($id, '$nick', '$epass',
					'$name', '$email', $permits)",
				$sqlid);
	if (affected_squash_rows($res) == 1) {
		return true;
	}
	return false;
}

function get_squash_user_id($nick, $sqlid) {
	$res = query_squash("
				SELECT id
				FROM people
				WHERE nick='$nick'",
				$sqlid);
	if (num_squash_rows($res) > 0) {
		$row = fetch_squash_row($res);
		return $row[0];
	}
	return -1;
}

function get_squash_user_permits($nick, $sqlid) {
	$res = query_squash("
				SELECT permits
				FROM people
				WHERE nick='$nick'",
				$sqlid);
	if (num_squash_rows($res) > 0) {
		$row = fetch_squash_row($res);
		return $row[0];
	}
	return 99; /* big number means less privileges */
}

function validate_squash_user($nick, $password, $sqlid) {
	$epass = crypt_squash_password($password, $sqlid);
	$res = query_squash("
				SELECT password
				FROM people
				WHERE nick='$nick'
				AND password='$epass'",
				$sqlid);
	if (num_squash_rows($res) > 0) {
		return true;
	}
	return false;
}

/*
	Squash Articles/Posts related functions.
	Add/edit/remove articles, validates authors for editing
*/

function is_post_author($post_id, $nick, $sqlid) {
	$res = query_squash("
				SELECT posts.id
				FROM people, posts
				WHERE people.nick='$nick'
				AND posts.id_author=people.id
				AND posts.id=$post_id",
				$sqlid);
	if (num_squash_rows($res) > 0) {
		return true;
	}
	return false;
}

function add_squash_article($nick, $title, $source, $topic, $summary,
							$content, $sqlid) {
	$user_id = get_squash_user_id($nick, $sqlid);
	$timestamp = date("Y-m-d H:i:s", time());
	$post_id = get_next_squash_id("posts", $sqlid);
	$res = query_squash("
				INSERT INTO posts
				VALUES ($post_id, '$title', $user_id, $topic,
					'$timestamp', '$source')",
				$sqlid);
	if (affected_squash_rows($res) != 1) {
		return false;
	}
	$res = query_squash("
				INSERT INTO posts_summary
				VALUES ($post_id, '$summary')",
				$sqlid);
	if (affected_squash_rows($res) != 1) {
		return false;
	}
	$res = query_squash("
				INSERT INTO posts_content
				VALUES ($post_id, '$content')",
				$sqlid);
	if (affected_squash_rows($res) != 1) {
		return false;
	}
	return true;
}

function update_squash_article($post_id, $title, $source, $topic, $summary,
							$content, $sqlid) {
	$timestamp = date("Y-m-d H:i:s", time());
	$res = query_squash("
				UPDATE posts
				SET title='$title',
					id_topic=$topic,
					td_stamp='$timestamp',
					source='$source'
				WHERE id=$post_id",
				$sqlid);
	if (affected_squash_rows($res) != 1) {
		return false;
	}
	$res = query_squash("
				UPDATE posts_summary
				SET data='$summary'
				WHERE id=$post_id",
				$sqlid);
	$res = query_squash("
				UPDATE posts_content
				SET data='$content'
				WHERE id=$post_id",
				$sqlid);
	return true;
}

function add_squash_comment($id, $nick, $comment, $sqlid) {
	$td = date("Y-m-d H:i:s", time());
	$aid = get_squash_user_id($nick, $sqlid);
	$cid = get_next_squash_id("comments", $sqlid);
	$res = query_squash("
				INSERT INTO comments
				VALUES ($cid, $id, $aid, '$td', '$comment')",
				$sqlid);
	if (affected_squash_rows($res) != 1) {
		return false;
	}
	return true;
}

function squash_article_title($post_id, $sqlid) {
	$res = query_squash("
				SELECT title
				FROM posts
				WHERE id=$post_id",
				$sqlid);
	if (num_squash_rows($res) > 0) {
		$row = fetch_squash_row($res);
		echo $row[0];
		return;
	}
	echo $MESSAGE_POSTNOTFOUND_ERROR;
}

function read_squash_article($post_id, $sqlid) {
	return query_squash("
				SELECT posts.id, posts.title,
					people.full_name, people.email,
					topics.name, posts.td_stamp,
					posts.source, posts_content.data
				FROM posts, topics, posts_content, people
				WHERE posts.id=$post_id
				AND posts.id=posts_content.id
				AND posts.id_topic=topics.id
				AND posts.id_author=people.id",
			$sqlid);
}

function squash_post_id($post_array) {
	echo $post_array[0];
}

function squash_post_title($post_array) {
	echo $post_array[1];
}

function squash_post_name($post_array) {
	echo $post_array[2];
}

function squash_post_email($post_array) {
	echo $post_array[3];
}

function squash_post_topic($post_array) {
	echo $post_array[4];
}

function squash_post_timedate($post_array) {
	echo $post_array[5];
}

function squash_post_source($post_array) {
	echo $post_array[6];
}

function squash_post_data($post_array) {
	echo $post_array[7];
}

/*
	Posts/Articles summary functions.
	These are here to help with the addition/edition of summaries
	(small content that describe the article/post)
*/

function get_squash_summary($sqlid) {
	return query_squash("
				SELECT posts.id, posts.title,
					people.full_name, people.email,
					topics.name, posts.td_stamp,
					posts.source, posts_summary.data
				FROM posts, topics, posts_summary, people
				WHERE posts.id=posts_summary.id
				AND posts.id_topic=topics.id
				AND posts.id_author=people.id
				ORDER BY td_stamp DESC LIMIT 10",
				$sqlid);
}

function squash_summary_data($post_id, $sqlid) {
	$res = query_squash("
				SELECT data
				FROM posts_summary
				WHERE id=$post_id",
				$sqlid);
	if (num_squash_rows($res) > 0) {
		$row = fetch_squash_row($res);
		echo $row[0];
		return;
	}
	echo $MESSAGE_SUMMARYNOTFOUND_ERROR;
}

/*
	Posts/articles comments functions.
	Do the entry and editing of comments following a post, these
	logically requiere that there is an existing post with an
	asigned ID
*/

function squash_comments_count($post_array, $sqlid) {
	$cres = query_squash("
				SELECT COUNT(id)
				FROM comments
				WHERE id_post=$post_array[0]",
				$sqlid);
	if (num_squash_rows($cres) > 0) {
		$row = fetch_squash_row($cres);
		echo $row[0];
		return;
	}
	echo "0";
}

function read_squash_comments($id, $sqlid) {
	return query_squash("
				SELECT posts.title, people.full_name,
					people.email, comments.td_stamp,
					comments.comment
				FROM posts, people, comments
				WHERE posts.id=$id
				AND posts.id=comments.id_post
				AND comments.id_author=people.id
				ORDER BY comments.id ASC",
				$sqlid);
}

function squash_comment_title($comment_array) {
	echo $comment_array[0];
}

function squash_comment_author($comment_array) {
	echo $comment_array[1];
}

function squash_comment_author_email($comment_array) {
	echo $comment_array[2];
}

function squash_comment_timedate($comment_array) {
	echo $comment_array[3];
}

function squash_comment_data($comment_array) {
	echo $comment_array[4];
}

/*
	HTML Helper functions.
	These are complimentary to the system and help to serve the lazy
	(and smart) developer that does not want to work twice ;)
	if you want to contribute something healthy to the system and don't
	know where to start, this is a good place!
*/

function show_squash_topic_selector($described, $selected, $sqlid) {
	$res = query_squash("
				SELECT *
				FROM topics",
				$sqlid);
	$count = num_squash_rows($res);
	echo "<SELECT NAME=\"topic\">\n";
	for ($i=0; $i<$count; $i++) {
		$row = fetch_squash_row($res);
		echo "<OPTION VALUE=\"$row[0]\"";
		if ($row[1] == $selected) {
			echo " SELECTED";
		}
		if ($described) {
			echo ">$row[2]</OPTION>\n";
		} else {
			echo ">$row[1]</OPTION>\n";
		}
	}
	echo "</SELECT>\n";
}

?>
